//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FourCCGUI.h"
#include "GeneralTools.h"
#include "MapGraph.h"
#include "SimpleRelaxtionModel.h"
#include "RelaxtionThread.h"
#include "RelaxtionModelInterface.h"
#include "SimpleRelaxtionModelWithTwoColorsInit.h"
#include "SimpleRelaxtionModelWithRandomInits.h"
#include "TieBrackingRelaxtionModelWithRandomInits.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "CSPIN"
#pragma link "CGAUGES"
#pragma resource "*.dfm"
TForm_4CC *Form_4CC;

const int TForm_4CC::defaultColorsTable[]={0x00676767,0x005C00B9,0x001D8B27,0x007E570E,0x009C9C63,
                                           0x00123911,
                                           0x009E2796,0x009DA422,0x00DD2622,0x001DE294,
                                           0x000E1A7E,0x00CD6C32,0x001FC8E0,0x00C8B937};
const int TForm_4CC::defaultColorsTableSize=14;

//---------------------------------------------------------------------------
__fastcall TForm_4CC::TForm_4CC(TComponent* Owner)
        : TForm(Owner),m_pixelSize(25)
{
}
//---------------------------------------------------------------------------
void TForm_4CC::drawMatrix(){

for (int i=0;i<m_currGridWidth;i++)
 for (int j=0;j<m_currGridHeight;j++){
  TColor pixelColor = m_matrix[i][j];
  if (pixelColor==0) pixelColor=Shape_BGColorSelection->Brush->Color;
  Image_Map->Canvas->Brush->Color=pixelColor;
  Image_Map->Canvas->FillRect(
        Rect(1+i*(m_pixelSize+1),
             1+j*(m_pixelSize+1),
             (i+1)*(m_pixelSize+1),
             (j+1)*(m_pixelSize+1)));
 }

}

void TForm_4CC::repaintGrid(){
m_pixelSize=CSpinEdit_PixelSize->Value;

Image_Map->Height=m_currGridHeight*(m_pixelSize+1)+1;
Image_Map->Width=m_currGridWidth*(m_pixelSize+1)+1;
Image_Map->Picture=0;

//Draw Grid
Image_Map->Canvas->Brush->Color = Shape_BGColorSelection->Brush->Color;
Image_Map->Canvas->Pen->Color =Shape_BGColorSelection->Pen->Color;
Image_Map->Canvas->Rectangle(Rect(0,0,Image_Map->Width,Image_Map->Height));
Image_Map->Canvas->FillRect(Rect(1,1,Image_Map->Width-1,Image_Map->Height-1));

Image_Map->Canvas->Pen->Color = Shape_GridColorSelection->Pen->Color;

for (int i=1;i<m_currGridWidth;i++){
 Image_Map->Canvas->MoveTo(i*(m_pixelSize+1),1);
 Image_Map->Canvas->LineTo(i*(m_pixelSize+1),Image_Map->Height-1);
 }

for (int i=1;i<m_currGridHeight;i++){
 Image_Map->Canvas->MoveTo(1,i*(m_pixelSize+1));
 Image_Map->Canvas->LineTo(Image_Map->Width-1,i*(m_pixelSize+1));
 }

drawMatrix();
}


void __fastcall TForm_4CC::SpeedButton_ClearClick(TObject *Sender)
{
SpeedButton_DrawMode->Down=true;
SpeedButton_DrawMode->Click();

delete []m_matrix;
m_currGridHeight=CSpinEdit_MapHeight->Value;
m_currGridWidth=CSpinEdit_MapWidth->Value;

m_matrix=new int*[m_currGridWidth];
for (int i=0;i<m_currGridWidth;i++)
 m_matrix[i]=new int[m_currGridHeight];

//Empty The Maxtix
for (int i=0;i<m_currGridWidth;i++)
 for (int j=0;j<m_currGridHeight;j++)
  m_matrix[i][j]=0;

repaintGrid();



}
//---------------------------------------------------------------------------


void __fastcall TForm_4CC::CSpinEdit_PixelSizeChange(TObject *Sender)
{
repaintGrid();
}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::FormCreate(TObject *Sender)
{
m_lastPaintY=-1;
m_lastPaintX=-1;
SpeedButton_Clear->Click();
SpeedButton_DrawMode->Click();

CGauge_Color1->ForeColor=defaultColorsTable[1]+70;
CGauge_Color1->BackColor=defaultColorsTable[1];

CGauge_Color2->ForeColor=defaultColorsTable[2]+16;
CGauge_Color2->BackColor=defaultColorsTable[2];

CGauge_Color3->ForeColor=defaultColorsTable[3]+10;
CGauge_Color3->BackColor=defaultColorsTable[3];

CGauge_Color4->ForeColor=defaultColorsTable[4]+70;
CGauge_Color4->BackColor=defaultColorsTable[4];




}
//---------------------------------------------------------------------------




void __fastcall TForm_4CC::Image_MapMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
X=(X-1)/(m_pixelSize+1);
Y=(Y-1)/(m_pixelSize+1);


if (X<0 || Y<0 || X>=m_currGridWidth || Y>=m_currGridHeight)
 return;



if (m_buttonPressed==mbLeft)
        m_matrix[X][Y]=Shape_Pixel->Brush->Color;
  else
        m_matrix[X][Y]=0;
TColor pixelColor = m_matrix[X][Y];
if (pixelColor==0) pixelColor=Shape_BGColorSelection->Brush->Color;
Image_Map->Canvas->Brush->Color=pixelColor;
Image_Map->Canvas->FillRect(
        Rect(1+X*(m_pixelSize+1),
             1+Y*(m_pixelSize+1),
             (X+1)*(m_pixelSize+1),
             (Y+1)*(m_pixelSize+1)));
ScrollBox_Unlimited_Canvas->DoubleBuffered=true;

}
//---------------------------------------------------------------------------


void __fastcall TForm_4CC::Image_MapMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
m_buttonPressed=Button;
Image_Map->OnMouseMove=Image_MapMouseMove;
Image_MapMouseMove(0,Shift,X,Y);


}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::Image_MapMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
Image_Map->OnMouseMove=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_LoadClick(TObject *Sender)
{


if (OpenPictureDialog->Execute()) {
        SpeedButton_DrawMode->Down=true;
        SpeedButton_DrawMode->Click();
        Image_Temp->Picture->LoadFromFile(OpenPictureDialog->FileName);
        CSpinEdit_MapWidth->Value=Image_Temp->Width;
        CSpinEdit_MapHeight->Value=Image_Temp->Height;
        SpeedButton_Clear->Click();
        for (int i=0;i<Image_Temp->Width;i++)
         for (int j=0;j<Image_Temp->Height;j++)
           m_matrix[i][j]=
                Image_Temp->Picture->Bitmap->Canvas->Pixels[i][j]==clWhite ?
                0 : Shape_Pixel->Brush->Color;
        repaintGrid();

}


}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_SaveClick(TObject *Sender)
{

if (SavePictureDialog->Execute()) {
        SpeedButton_DrawMode->Down=true;
        SpeedButton_DrawMode->Click();

        Image_Temp->Picture=0;
        Image_Temp->AutoSize=false;
        Image_Temp->Width=m_currGridWidth;
        Image_Temp->Height=m_currGridHeight;
        for (int i=0;i<m_currGridWidth;i++)
         for (int j=0;j<m_currGridHeight;j++)
           Image_Temp->Canvas->Pixels[i][j]=m_matrix[i][j]==0 ? clWhite : m_matrix[i][j];

        Image_Temp->AutoSize=true;
        Image_Temp->Picture->Bitmap->SaveToFile(SavePictureDialog->FileName);
        }

}

//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_ColorClick(TObject *Sender)
{
if (ColorDialog->Execute())
        Shape_Pixel->Brush->Color=ColorDialog->Color;
}
//---------------------------------------------------------------------------


void __fastcall TForm_4CC::SpeedButton_DrawModeClick(TObject *Sender)
{
SpeedButton_Pause->Down=true;
SpeedButton_Pause->Click();
if (m_relaxtionThread!=0 && !m_relaxtionThread->isFinished() ) m_relaxtionThread->Suspend();
//delete m_relaxtionThread;
m_relaxtionThread=0;

L_NumberOfVertices->Caption="####";
L_NumberOfIteration->Caption="####";
L_unambiguousCount->Caption="####";

ListView_Regions->Clear();
if (m_lastPaintX!=-1){  //Restore Area
        GeneralTools::fill(m_matrix,makeSelectionColor(m_oldColor),m_oldColor,
         m_lastPaintX,m_lastPaintY,m_currGridWidth,m_currGridHeight);
        repaintGrid();
}
if (m_mapGraph!=0) {
        m_mapGraph->restoreDisplayColors(m_matrix,Shape_Pixel->Brush->Color);
        delete m_mapGraph;
        m_mapGraph=0;
        repaintGrid();
        }



Image_Map->OnMouseDown=Image_MapMouseDown;
Image_Map->OnMouseMove=0;
Image_Map->OnMouseUp= Image_MapMouseUp;
}
//---------------------------------------------------------------------------


int TForm_4CC::makeSelectionColor(int col){

 return col+0x161616;

}


void __fastcall TForm_4CC::OnMouseSelect(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
Timer_RestoreSelectedContry->Enabled=true;

X=(X-1)/(m_pixelSize+1);
Y=(Y-1)/(m_pixelSize+1);

if (X<0 || Y<0 || X>=m_currGridWidth || Y>=m_currGridHeight)
        return;

if (m_matrix[X][Y]!=Shape_Pixel->Brush->Color && m_matrix[X][Y]!=makeSelectionColor(m_oldColor)) {

  if (m_lastPaintX!=-1)  //Restore Area
        GeneralTools::fill(m_matrix,makeSelectionColor(m_oldColor),m_oldColor,
         m_lastPaintX,m_lastPaintY,m_currGridWidth,m_currGridHeight);

  m_oldColor=m_matrix[X][Y];
  m_lastPaintX=X;
  m_lastPaintY=Y;
  GeneralTools::fill(m_matrix,m_matrix[X][Y],makeSelectionColor(m_oldColor),X,Y,m_currGridWidth,m_currGridHeight);
  repaintGrid();


  int vertexNumber=m_mapGraph->getVertexOnGraphicXY(X,Y);
  ListView_Regions->ItemIndex=vertexNumber;


}



}
void TForm_4CC::updateRegionsDisplay(){
  ImageList_RegionsColors->Clear();
  int oldItemIndex=ListView_Regions->ItemIndex;
  ListView_Regions->Clear();

  for (int i=0;i<m_mapGraph->vertexCount();i++){
          Image_Temp->Picture=0;
          Image_Temp->AutoSize=false;
          Image_Temp->Width=ImageList_RegionsColors->Width;
          Image_Temp->Height=ImageList_RegionsColors->Height;
          Image_Temp->Canvas->Brush->Color= m_colorRemapTable[i];
          Image_Temp->Canvas->FillRect(Rect(0,0,Image_Temp->Width,Image_Temp->Height));
          ImageList_RegionsColors->AddMasked(Image_Temp->Picture->Bitmap,0);
          ListView_Regions->Items->Add();

          ListView_Regions->Items->Item[ListView_Regions->Items->Count-1]
                ->Caption="Region "+IntToStr(i)+" "+m_mapGraph->neighborsOfVertex(i);
          ListView_Regions->Items->Item[ListView_Regions->Items->Count-1]
                ->ImageIndex=ImageList_RegionsColors->Count-1;



          Image_Temp->AutoSize=true;

          }

ListView_Regions->ItemIndex=oldItemIndex;
ListView_Regions->DoubleBuffered=true;
}


//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_SelectModeClick(TObject *Sender)
{
if (m_mapGraph!=0) return;

delete m_mapGraph;
m_mapGraph=new MapGraph(m_matrix,m_currGridWidth,m_currGridHeight);

delete m_colorRemapTable;
m_colorRemapTable=new int[m_mapGraph->vertexCount()];

for (int i=0;i<m_mapGraph->vertexCount();i++)
 m_colorRemapTable[i]=
        (i<defaultColorsTableSize) ?
        defaultColorsTable[i] :
        Shape_BaseColorOfRegion->Brush->Color+210*i;

m_mapGraph->updateDisplayColors(m_matrix,m_colorRemapTable);

repaintGrid();
L_NumberOfVertices->Caption=IntToStr(m_mapGraph->vertexCount());

updateRegionsDisplay();

Image_Map->OnMouseDown=0;
Image_Map->OnMouseMove=OnMouseSelect;
Image_Map->OnMouseUp= 0;
m_lastPaintX=-1;
m_lastPaintY=-1;
}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::Timer_RestoreSelectedContryTimer(
      TObject *Sender)
{
int x= Mouse->CursorPos.x-Image_Map->ClientToScreen(Point(0,0)).x;
int y= Mouse->CursorPos.y-Image_Map->ClientToScreen(Point(0,0)).y;
if (x<0 || y<0 || x>Image_Map->Width || y>Image_Map->Height){
               if (m_lastPaintX!=-1){  //Restore Area
                        GeneralTools::fill(m_matrix,makeSelectionColor(m_oldColor),m_oldColor,
                         m_lastPaintX,m_lastPaintY,m_currGridWidth,m_currGridHeight);
                        repaintGrid();
                        m_lastPaintX=-1;
                        Timer_RestoreSelectedContry->Enabled=false;
                        ListView_Regions->ItemIndex=-1;
                }
        }

}
//---------------------------------------------------------------------------



void __fastcall TForm_4CC::SpeedButton_ZoomInClick(TObject *Sender)
{
CSpinEdit_PixelSize->Value=CSpinEdit_PixelSize->Value+1;
}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_ZoomOutClick(TObject *Sender)
{
CSpinEdit_PixelSize->Value=CSpinEdit_PixelSize->Value-1;
}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_StartClick(TObject *Sender)
{
SpeedButton_DrawMode->Click();
SpeedButton_SelectMode->Down=true;
SpeedButton_SelectMode->Click();
SpeedButton_SelectMode->Enabled=false;
if (m_mapGraph->vertexCount()==1) return;


Timer_DisplayProgress->Enabled=true;
Timer_DisplayProgress->Interval=600;
GroupBox_penMode->Enabled=false;
SpeedButton_DrawMode->Enabled=false;

if (m_relaxtionThread!=0) {
        SpeedButton_Pause->Click();
        SpeedButton_DrawMode->Down=true;
        SpeedButton_DrawMode->Click();
}





RelaxtionModelInterface* rel;

 switch (ComboBox_RelaxtionSelection->ItemIndex) {
         case 0 :    //Simple w/one init
            rel=new SimpleRelaxtionModel(m_mapGraph);
            break;
         case 1 :    //Simple w/two init
            rel=new SimpleRelaxtionModelWithTwoColorsInit(m_mapGraph);
            break;
         case 2 :  //Simple w/random inits
            rel=new SimpleRelaxtionModelWithRandomInits(m_mapGraph);
         case 3:
            rel=new TieBrackingRelaxtionModelWithRandomInits(m_mapGraph);
            break;
         default :
            rel=new SimpleRelaxtionModel(m_mapGraph);
 }


 m_relaxtionThread=new RelaxtionThread(rel);


m_relaxtionThread->Resume();

}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::SpeedButton_PauseClick(TObject *Sender)
{
Timer_DisplayProgress->Enabled=false;
GroupBox_penMode->Enabled=true;
SpeedButton_SelectMode->Enabled=true;
SpeedButton_DrawMode->Enabled=true;

if (m_relaxtionThread!=0 && !m_relaxtionThread->isFinished())
        m_relaxtionThread->Suspend();

}
//---------------------------------------------------------------------------


void __fastcall TForm_4CC::ListView_RegionsChange(TObject *Sender,
      TListItem *Item, TItemChange Change)
{
int item = ListView_Regions->ItemIndex;
GroupBox_RegionInfo->DoubleBuffered=true;

if (item==-1) {
        CGauge_Color1->Progress=0;
        CGauge_Color2->Progress=0;
        CGauge_Color3->Progress=0;
        CGauge_Color4->Progress=0;
        L_RegionID->Caption="####";
        L_NeighborsList->Caption="{##,##,##}";
}
 else
 {
        CGauge_Color1->Progress=m_mapGraph->getVertexColorProbability(item,0)*100;
        CGauge_Color2->Progress=m_mapGraph->getVertexColorProbability(item,1)*100;
        CGauge_Color3->Progress=m_mapGraph->getVertexColorProbability(item,2)*100;
        CGauge_Color4->Progress=m_mapGraph->getVertexColorProbability(item,3)*100;
        L_RegionID->Caption=IntToStr(item);
        L_NeighborsList->Caption=m_mapGraph->neighborsOfVertex(item);
 }


}
//---------------------------------------------------------------------------

void __fastcall TForm_4CC::Timer_DisplayProgressTimer(TObject *Sender)
{
if (m_relaxtionThread->isFinished()) {
SpeedButton_Pause->Down=true;
SpeedButton_Pause->Click();

}


delete m_colorRemapTable;
m_colorRemapTable=new int[m_mapGraph->vertexCount()];
if (Timer_DisplayProgress->Interval<2000)
        Timer_DisplayProgress->Interval+=300;
L_NumberOfIteration->Caption=IntToStr(m_relaxtionThread==0 ? -1 :m_relaxtionThread->getIterationCounter());
int numberOfunAmbig=0;
for (int i=0;i<m_mapGraph->vertexCount();i++){
 unsigned char red=0;
 unsigned char green=0;
 unsigned char blue=0;
 unsigned char intes=0;

 for (int c=1;c<=m_mapGraph->getWantedColorsCount();c++){
  if (m_mapGraph->getVertexColorProbability(i,c-1)>0.95) numberOfunAmbig++;
  red=red+
  (int)((double)(0x000000FF&defaultColorsTable[c])*m_mapGraph->getVertexColorProbability(i,c-1));
  green=green+
  (int)((double)((0x0000FF00&defaultColorsTable[c])/256)*m_mapGraph->getVertexColorProbability(i,c-1));
  blue=blue+
  (int)((double)((0x00FF0000&defaultColorsTable[c])/(256*256))*m_mapGraph->getVertexColorProbability(i,c-1));
  }

 unsigned int color=red+green*256+blue*256*256;

 m_colorRemapTable[i]=color;
}


m_mapGraph->updateDisplayColors(m_matrix,m_colorRemapTable);
repaintGrid();
updateRegionsDisplay();
L_unambiguousCount->Caption=IntToStr(numberOfunAmbig);
if (numberOfunAmbig==m_mapGraph->vertexCount()) SpeedButton_Pause->Click();


}
//---------------------------------------------------------------------------


void __fastcall TForm_4CC::ComboBox_RelaxtionSelectionChange(
      TObject *Sender)
{
SpeedButton_Pause->Click();
SpeedButton_DrawMode->Down=true;
SpeedButton_DrawMode->Click();
}
//---------------------------------------------------------------------------

