
//---------------------------------------------------------------------------


#pragma hdrstop

#include "MapGraph.h"
#include "GeneralTools.h"
const int MapGraph::MAX_BOARDER_WIDTH=2;




AnsiString MapGraph::neighborsOfVertex(int vertex){
AnsiString s="{";
for (long i=0;i<m_vertexCount;i++)
 if (i!=vertex && m_neighborsMatrix[i][vertex]) s=s+(s=="{" ? IntToStr(i) : (AnsiString)(","+IntToStr(i)));

return s+"}";
}


MapGraph::~MapGraph(){

for (int i=0;i<m_graphcWidth;i++)
  delete m_graphicMatrix[i];

for (int i=0;i<m_vertexCount;i++)
  delete m_vertexColorProbability[i];

for (int i=0;i<m_vertexCount;i++)
   delete m_neighborsMatrix[i];

delete m_vertexColorProbability;
delete m_graphicMatrix;
delete m_neighborsMatrix;
}



MapGraph::MapGraph(int **graphicDrawnMap,int width,int height,int wantedColorsCount){
 m_graphcHeight=height;
 m_graphcWidth=width;
 m_graphicMatrix=new int*[width];

 for (int i=0;i<width;i++)
  m_graphicMatrix[i]=new int[height];

 for (int i=0;i<width;i++)
  for (int j=0;j<height;j++)
   m_graphicMatrix[i][j]=(graphicDrawnMap[i][j] == 0 ? 0 : -1);
  //my matrix contans -1 iff it is an edge.

 //now I have to build the graph...
 m_vertexCount=0;

 for (int i=0;i<width;i++)
   for (int j=0;j<height;j++)
    if (m_graphicMatrix[i][j]==0) { //found new vertex
      m_vertexCount++;
      GeneralTools::fill(m_graphicMatrix,0,m_vertexCount,i,j,width,height);
    }

 //Create an empty graph (with no edges)
 m_vertexColorProbability=new double*[m_vertexCount];
 m_wantedColorsCount=wantedColorsCount;
 for (int i=0;i<m_vertexCount;i++)
  m_vertexColorProbability[i]=new double[wantedColorsCount];

 for (int i=0;i<m_vertexCount;i++)
  for (int j=0;j<m_wantedColorsCount;j++)
   m_vertexColorProbability[i][j]=(1/(double)m_wantedColorsCount);
   //each vertex get initial value of 0.25 for each of the 4 colors

 m_neighborsMatrix=new bool*[m_vertexCount];
 for (int i=0;i<m_vertexCount;i++)
   m_neighborsMatrix[i]=new bool[m_vertexCount];

 for (int i=0;i<m_vertexCount;i++)
  for (int j=0;j<m_vertexCount;j++)
    m_neighborsMatrix[i][j]=i==j ? true : false;


 //add edges to the graph..
 recognizeNeighbors();

}


//Note: graphicDrawnMap is changed!!!
void MapGraph::restoreDisplayColors(int **graphicDrawnMap,int borderColor){
for (int i=0;i<m_graphcWidth;i++)
 for (int j=0;j<m_graphcHeight;j++)
  graphicDrawnMap[i][j]=m_graphicMatrix[i][j]==-1 ? borderColor : 0;

}

//This proc re-maps the colors !=-1 of m_graphicMatrix to other display matrix
//Note: graphicDrawnMap is changed!!!
//PRE: size of colorTable is at least m_vertexCount
void MapGraph::updateDisplayColors(int **graphicDrawnMap,int* colorTable){
for (int i=0;i<m_graphcWidth;i++)
 for (int j=0;j<m_graphcHeight;j++)
  if (m_graphicMatrix[i][j]!=-1)
     graphicDrawnMap[i][j]=colorTable[m_graphicMatrix[i][j]-1];
}



void MapGraph::assignNeighbors(int n1,int n2){
m_neighborsMatrix[n1-1][n2-1]=true;
m_neighborsMatrix[n2-1][n1-1]=true;
}


void MapGraph::recognizeNeighbors(){
 //Searchs for the patters: Number(Boarder)*Number
 // Number
 // (Boarder)*
 // Number
for (int i=0;i<m_graphcWidth;i++)
 for (int j=0;j<m_graphcHeight;j++)
   if (m_graphicMatrix[i][j]==-1) {
     int yTop=j;
     for (;yTop>=0 && m_graphicMatrix[i][yTop]==-1;yTop--);
     int yTopPos=yTop;
     yTop=yTop == -1 ? -1 : m_graphicMatrix[i][yTop];

     int yButtom=j;
     for (;yButtom<m_graphcHeight && m_graphicMatrix[i][yButtom]==-1;yButtom++);
     int yButtomPos=yButtom;
     yButtom=yButtom == m_graphcHeight ? -1 : m_graphicMatrix[i][yButtom];

     if (yButtom!=-1 && yTop!=-1 && yButtomPos-yTopPos<MAX_BOARDER_WIDTH+2
          && yButtom!=yTop) assignNeighbors(yButtom,yTop);

     int xLeft=i;
     for (;xLeft>=0 && m_graphicMatrix[xLeft][j]==-1;xLeft--);
     int xLeftPos=xLeft;
     xLeft=xLeft == -1 ? -1 : m_graphicMatrix[xLeft][j];

     int xRight=i;
     for (;xRight<m_graphcWidth && m_graphicMatrix[xRight][j]==-1;xRight++);
     int xRightPos=xRight;
     xRight=xRight == m_graphcWidth ? -1 : m_graphicMatrix[xRight][j];

     if (xRight!=-1 && xLeft!=-1 && xRightPos-xLeftPos<MAX_BOARDER_WIDTH+2
          && xRight!=xLeft) assignNeighbors(xLeft,xRight);

   }

}


int MapGraph::vertexCount(){
return m_vertexCount;
}

//--- Getters and Setters:

void MapGraph::setVertexColorProbability(int vertexID,int color,double prob){
m_vertexColorProbability[vertexID][color]=prob;
}

double MapGraph::getVertexColorProbability(int vertexID,int color){
return m_vertexColorProbability[vertexID][color];
}


int MapGraph::getVertexOnGraphicXY(int x,int y){
 return m_graphicMatrix[x][y]-1; //returns the vertex number (= color -1)
}


int MapGraph::getWantedColorsCount(){
 return m_wantedColorsCount;
}

bool MapGraph::isNeighbors(int n1,int n2){
 return m_neighborsMatrix[n1][n2];
}


//---------------------------------------------------------------------------

#pragma package(smart_init)
