//---------------------------------------------------------------------------


#pragma hdrstop

#include "SimpleRelaxtionModel.h"
#include "MapGraph.h"


void SimpleRelaxtionModel::initGraph(){
m_graph->setVertexColorProbability(0,0,1);
for (int i=1;i<m_graph->getWantedColorsCount();i++)
        m_graph->setVertexColorProbability(0,i,0);
}


SimpleRelaxtionModel::SimpleRelaxtionModel(MapGraph* graph) :RelaxtionModelInterface(graph) {
initGraph();
m_tempProbabilities=new double*[m_graph->vertexCount()];
for (int i=0;i<m_graph->vertexCount();i++)
  m_tempProbabilities[i]=new double[m_graph->getWantedColorsCount()];


}

void SimpleRelaxtionModel::relax(){
for (int i=0;i<m_graph->vertexCount();i++)
 for (int j=0;j<m_graph->getWantedColorsCount();j++)
  m_tempProbabilities[i][j]=m_graph->getVertexColorProbability(i,j);

for (int i=0;i<m_graph->vertexCount();i++){
 double normalDenum=0;
 for (int m=0;m<m_graph->getWantedColorsCount();m++)
   normalDenum+=m_tempProbabilities[i][m]*support(i,m);

 for (int lambda=0;lambda<m_graph->getWantedColorsCount();lambda++)
  m_graph->setVertexColorProbability(i,lambda,
                                     (m_tempProbabilities[i][lambda]*support(i,lambda))/normalDenum);


}


}


double SimpleRelaxtionModel::competability(int vertex1,int vertex2,int color1,int color2){
if (vertex1==vertex2) return 0;
if (color1==color2) return m_graph->isNeighbors(vertex1,vertex2) ? -1 : 0;
  else return m_graph->isNeighbors(vertex1,vertex2) ? 1 : 0;
}

double SimpleRelaxtionModel::support(int vertexID,int color){
double sum=0;
for (int j=0;j<m_graph->vertexCount();j++)
  for (int m=0;m<m_graph->getWantedColorsCount();m++)
   sum+=competability(vertexID,j,color,m)*m_tempProbabilities[j][m];
return sum;
}

double abs(double d){
if (d<0) return -d;
 else return d;
}

bool SimpleRelaxtionModel::is_finished(){
static int timesSame=0;
double delta=0;
for (int i=0;i<m_graph->vertexCount();i++)
 for (int j=0;j<m_graph->getWantedColorsCount();j++)
  delta=delta+abs(m_tempProbabilities[i][j]-m_graph->getVertexColorProbability(i,j));

if (delta<0.7) timesSame++;
 else timesSame=0;

return delta<0.5 && timesSame>10;

}



//---------------------------------------------------------------------------

#pragma package(smart_init)
